@@ -36,15 +36,20 @@
 #include "NMix_PosterMeanMixParam.h"
 
 #include "GLMM.h"
+#include "GLMM_Deviance.h"
+
 #include "GLMM_linear_predictors.h"
 //#include "GLMM_scale_ZitZi.h"
 #include "GLMM_create_SZitZiS.h"
 #include "GLMM_create_XtX.h"
+#include "GLMM_create_ZS.h"
+#include "GLMM_dY_meanY.h"
 
 #include "GLMM_updateVars_eps.h"
 #include "GLMM_updateHyperVars_eps.h"
 #include "GLMM_updateFixEf.h"
 #include "GLMM_updateRanEf.h"
+#include "GLMM_updateRanEf_QR.h"
 
 
 #ifdef __cplusplus
@@ -61,24 +66,26 @@ extern "C" {
 //                                   ...
 //                                   response R_c-1 for cluster 0, ..., response R_c-1 for cluster I-1
 //
-//  R_c[1]:                  number of continuous response variables
-//
 //  Y_d[]:                   discrete integer response
 //
-//  R_d[1]:                  number of discrete response variables
-//                           * in the following R = R_c + R_d
+//  keepChain_nMCMC_R_cd_dist[]:
+//                           0: keepChain = 0/1 indicating whether we should keep chains of "optional" parameters
+//                              keepChain[0] = keep sampled values of random effects?
+//                           +1: nMCMC[4] = length of MCMC, see NMix_MCMC.h
+//                           +4: R_c = number of continuous response variables
+//                           +1: R_d = number of discrete response variables
+//                               * in the following R = R_c + R_d
+//                           +1: dist[R]:                 distribution of each response
+//                               * see GLMM::_GLMM_dist in GLMM.h
 //
-//  dist[R]:                 distribution of each response
-//                           * see GLMM::_GLMM_dist in GLMM.h
-//
-//  I[1]:                    number of clusters
-//
-//  n[R*I]:                  numbers of observations for each response and each cluster
-//                           * ordering: n[response 0, cluster 0], n[response 0, cluster 1], ..., n[response 0, cluster I-1],
-//                                       n[response 1, cluster 0], n[response 1, cluster 1], ..., n[response 1, cluster I-1],
-//                                       ...
-//                                       n[response R-1, cluster 0], n[response R-1, cluster 1], ..., n[response R-1, cluster I-1], 
-//                           * in the following N = sum(n) (N = total number of observations)
+//  I_n[1 + R*I]:            I_n[0] = I = number of groups of correlated observations
+//                           I_n[1,...] = n =
+//                             numbers of observations for each response and each cluster
+//                             * ordering: n[response 0, cluster 0], n[response 0, cluster 1], ..., n[response 0, cluster I-1],
+//                                         n[response 1, cluster 0], n[response 1, cluster 1], ..., n[response 1, cluster I-1],
+//                                         ...
+//                                         n[response R-1, cluster 0], n[response R-1, cluster 1], ..., n[response R-1, cluster I-1], 
+//                             * in the following N = sum(n) (N = total number of observations)
 //
 //  X[]                      covariate matrices for fixed effects (without a column of ones for intercept)
 //                           * ordering of covariates: response 0 for observation (0, 0), response 0 for observation (0, 1), 
@@ -90,12 +97,6 @@ extern "C" {
 //                                                     response R-1 for observation (I-1, n[0, I-1])
 //  XtX[]                    lower triangles of matrices t(X_s) %*% X_s, where X_s is the design matrix of the fixed effects 
 //                           (including possibly column of ones for a fixed intercept) for response s (s=0,...,R-1)
-//                                                     
-//  p[R]:                    numbers of fixed effects covariates for each response corresponding to X matrices
-//                           * these numbers do not count possible intercepts
-//
-//  fixedIntcpt[R]:          0/1 indicating whether there is intercept included in beta vector for each response
-//                           * in the following l_beta = sum(p) + sum(fixedIntcpt)
 //
 //  Z[]:                     covariate matrices for random effects (without a column of ones for intercept)
 //                           * ordering the same as for X matrix
@@ -109,23 +110,22 @@ extern "C" {
 //                           OUTPUT: lower triangles of matrices S_s %*% t(Z_s[i]) %*% Z_s[i] %*% S_s,
 //                                   where S is the diagonal matrix with scale_b on a diagonal  
 //
-//  q[R]:                    numbers of random effect covariates for each response corresponding to Z matrices
-//                           * these numbers do not count random intercept
-//
-//  randIntcpt[R]:           0/1 indicating whether there is random intercept included for each response
-//                           * in the following dim_b = sum(q) + sum(randIntcpt)
-//
-//  shiftScale_b[2*sum(q)]:  shift and scale used for random effects to improve mixing
-//
-//
-//  MCMC PARAMETERS
-//  =============== 
-//  nMCMC[4]:            length of MCMC, see NMix_MCMC.h
-//
-//  keepChain[]:         0/1 indicating whether we should keep chains of "optional" parameters
-//                       keepChain[0] = keep sampled values of random effects?
+//  p_fI_q_rI[4*R]:          p_fI_q_rI[0,...,R-1]     = p[R]: 
+//                                       numbers of fixed effects covariates for each response corresponding to X matrices
+//                                       * these numbers do not count possible intercepts
+//                           p_fI_q_rI[R,...,2*R-1]   = fixedIntcpt[R]: 
+//                                       0/1 indicating whether there is intercept included in beta vector for each response
+//                                       * in the following l_beta = sum(p) + sum(fixedIntcpt)
+//                           p_fI_q_rI[2*R,...,3*R-1] = q[R]:
+//                                       numbers of random effect covariates for each response corresponding to Z matrices
+//                                       * these numbers do not count random intercept
+//                           p_fI_q_rI[3*R,...,4*R-1] = randIntcpt[R]:             
+//                                       0/1 indicating whether there is random intercept included for each response
+//                                       * in the following dim_b = sum(q) + sum(randIntcpt)
 //
+//  shiftScale_b[2*dim_b]:  shift and scale used for random effects to improve mixing
 //
+////
 //  PARAMETERS OF THE PRIOR DISTRIBUTION
 //  ====================================
 //
@@ -220,6 +220,9 @@ extern "C" {
 //  r_b[I]:                   INPUT:  initial allocations for random effects
 //                           OUTPUT:  last values of allocations
 //
+//  r_b_first[I]              INPUT:  whatsever
+//                           OUTPUT:  value of r_b corresponding to the first kept (not burn-in, after thinning loop) MCMC iteration
+//
 //
 //  INITIAL AND LAST SAMPLED VALUES RELATED TO REGRESSION
 //  =====================================================
@@ -230,6 +233,10 @@ extern "C" {
 //                             OUTPUT:  last sampled value
 //
 //
+//  b_first[dim_b*I]:           INPUT:  whatsever
+//                             OUTPUT:  value of b corresponding to the first kept (not burn-in, after thinning loop) MCMC iteration
+//
+//
 //  CHAINS FOR QUANTITIES RELATED TO THE DISTRIBUTION OF RESIDUALS
 //  ==============================================================
 //  chsigma_eps[]
@@ -269,6 +276,14 @@ extern "C" {
 //  chb
 //
 //  
+//  OTHER CHAINS 
+//  =============
+//  chGLMMLogL[]               chain with the marginal (random effects integrated out) log-likelihood at each iteration of MCMC
+//                             - the marginal log-likelihood is obtained using the Laplacian approximation
+//
+//  chLogL[]                   chain with the conditional (given random effects) log-likelihood at each iteration of MCMC
+//
+//
 //  PERFORMANCE OF MCMC
 //  ====================
 //  naccept_beta[R_c + R_d]    numbers of accepted beta's (in Metropolis-Hastings step)
@@ -287,6 +302,10 @@ extern "C" {
 //
 //  pm_eta_random
 //
+//  pm_meanY
+//
+//  pm_stres
+//
 //  pm_b
 //
 //  pm_w
@@ -299,6 +318,8 @@ extern "C" {
 //
 //  pm_Li_b
 //
+//  pm_indGLMMLogL[I]: posterior means of marginal (random effects integrated out) log-likelihood for each cluster
+//
 //  pm_indLogL[I]:     posterior means of log-(conditional given b) likelihood for each cluster
 //
 //  pm_indLogpb[I]:    posterior means of log-density of random effects for each cluster
@@ -322,24 +343,14 @@ extern "C" {
 //  err[1]:            error flag
 //
 void
-GLMM_MCMC(double*       Y_c,                               // this is in fact const, not declared as const to be able to use **
-          const int*    R_c,   
-          int*          Y_d,                               // this is in fact const, not declared as const to be able to use **
-          const int*    R_d,  
-          const int*    dist,                 
-          const int*    I,                  
-          int*          n,                                  // this is in fact const, not declared as const to be able to use **
+GLMM_MCMC(double*       Y_c,                                // this is in fact const, not declared as const to be able to use **
+          int*          Y_d,                                // this is in fact const, not declared as const to be able to use **
+          const int*    keepChain_nMCMC_R_cd_dist,  
+          int*          I_n,                                // this is in fact const, not declared as const to be able to use **
           const double* X, 
-          //const double* XtX,                              // REMOVED ON 21/10/2009,  matrices XtX are computed directly in C++ code from X
-          const int*    p,                  
-          const int*    fixedIntcpt,
           double*       Z,                                  // this is in fact const, not declared as const to be able to use **
-          //double*       SZitZiS,                          // REMOVED ON 20/10/2009,  matrices SZitZiS are computed directly in C++ code
-          const int*    q,                  
-          const int*    randIntcpt,   
+          const int*    p_fI_q_rI,
           const double* shiftScale_b,
-          const int*    nMCMC,
-          const int*    keepChain,
           const double* priorDouble_eps,
           const int*    priorInt_b,           
           const double* priorDouble_b,
@@ -356,8 +367,10 @@ GLMM_MCMC(double*       Y_c,                               // this is in fact co
           double* Li_b,
           double* gammaInv_b,    
           int*    r_b,
+          int*    r_b_first,
           double* beta,          
           double* b, 
+          double* b_first,
           double* chsigma_eps,   
           double* chgammaInv_eps,
           int*    chK_b,            
@@ -373,16 +386,21 @@ GLMM_MCMC(double*       Y_c,                               // this is in fact co
           double* chCorrData_b,
           double* chbeta,        
           double* chb,
+          double* chGLMMLogL,
+          double* chLogL,
           int*    naccept_beta,
           int*    naccept_b,
           double* pm_eta_fixed,
           double* pm_eta_random,
+          double* pm_meanY,
+          double* pm_stres,
           double* pm_b,
           double* pm_w_b,         
           double* pm_mu_b,        
           double* pm_Q_b,              
           double* pm_Sigma_b,      
           double* pm_Li_b,
+          double* pm_indGLMMLogL,
           double* pm_indLogL,
           double* pm_indLogpb,
           int*    sum_Ir_b,
